@@ -14,8 +14,7 @@ low-maintenance self-hosted email.
 
 	mox [-config config/mox.conf] ...
 	mox serve
-	mox quickstart user@domain
-	mox restart
+	mox quickstart [-existing-webserver] [-hostname host] user@domain [user | uid]
 	mox stop
 	mox setaccountpassword address
 	mox setadminpassword
@@ -41,6 +40,8 @@ low-maintenance self-hosted email.
 	mox config domain add domain account [localpart]
 	mox config domain rm domain
 	mox config describe-sendmail >/etc/moxsubmit.conf
+	mox config printservice >mox.service
+	mox example [name]
 	mox checkupdate
 	mox cid cid
 	mox clientconfig domain
@@ -83,26 +84,42 @@ requested, other TLS certificates are requested on demand.
 
 Quickstart generates configuration files and prints instructions to quickly set up a mox instance.
 
-Quickstart prints initial admin and account passwords, configuration files, DNS
-records you should create, instructions for setting correct user/group and
-permissions, and if you run it on Linux it prints a systemd service file.
+Quickstart writes configuration files, prints initial admin and account
+passwords, DNS records you should create. If you run it on Linux it writes a
+systemd service file and prints commands to enable and start mox as service.
 
-	usage: mox quickstart user@domain
+The user or uid is optional, defaults to "mox", and is the user or uid/gid mox
+will run as after initialization.
 
-# mox restart
+Quickstart assumes mox will run on the machine you run quickstart on and uses
+its host name and public IPs. On many systems the hostname is not a fully
+qualified domain name, but only the first dns "label", e.g. "mail" in case of
+"mail.example.org". If so, quickstart does a reverse DNS lookup to find the
+hostname, and as fallback uses the label plus the domain of the email address
+you specified. Use flag -hostname to explicitly specify the hostname mox will
+run on.
 
-Restart mox after validating the configuration file.
+Mox is by far easiest to operate if you let it listen on port 443 (HTTPS) and
+80 (HTTP). TLS will be fully automatic with ACME with Let's Encrypt.
 
-Restart execs the mox binary, which have been updated. Restart returns after
-the restart has finished. If you update the mox binary, keep in mind that the
-validation of the configuration file is done by the old process with the old
-binary. The new binary may report a syntax error. If you update the binary, you
-should use the "config test" command with the new binary to validate the
-configuration file.
+You can run mox along with an existing webserver, but because of MTA-STS and
+autoconfig, you'll need to forward HTTPS traffic for two domains to mox. Run
+"mox quickstart -existing-webserver ..." to generate configuration files and
+instructions for configuring mox along with an existing webserver.
 
-Like stop, existing connections get a 3 second period for graceful shutdown.
+But please first consider configuring mox on port 443. It can itself serve
+domains with HTTP/HTTPS, including with automatic TLS with ACME, is easily
+configured through both configuration files and admin web interface, and can act
+as a reverse proxy (and static file server for that matter), so you can forward
+traffic to your existing backend applications. Look for "WebHandlers:" in the
+output of "mox config describe-domains" and see the output of "mox example
+webhandlers".
 
-	usage: mox restart
+	usage: mox quickstart [-existing-webserver] [-hostname host] user@domain [user | uid]
+	  -existing-webserver
+	    	use if a webserver is already running, so mox won't listen on port 80 and 443; you'll have to provide tls certificates/keys, and configure the existing webserver as reverse proxy, forwarding requests to mox.
+	  -hostname string
+	    	hostname mox will run on, by default the hostname of the machine quickstart runs on; if specified, the IPs for the hostname are configured for the public listener
 
 # mox stop
 
@@ -390,6 +407,22 @@ Describe configuration for mox when invoked as sendmail.
 
 	usage: mox config describe-sendmail >/etc/moxsubmit.conf
 
+# mox config printservice
+
+Prints a systemd unit service file for mox.
+
+This is the same file as generated using quickstart. If the systemd service file
+has changed with a newer version of mox, use this command to generate an up to
+date version.
+
+	usage: mox config printservice >mox.service
+
+# mox example
+
+List available examples, or print a specific example.
+
+	usage: mox example [name]
+
 # mox checkupdate
 
 Check if a newer version of mox is available.
