@@ -300,8 +300,8 @@ func (c *conn) xsanity(err error, format string, args ...any) {
 
 type msgseq uint32
 
-// ListenAndServe starts all imap listeners for the configuration, in new goroutines.
-func ListenAndServe() {
+// Listen initializes all imap listeners for the configuration, and stores them for Serve to start them.
+func Listen() {
 	for name, listener := range mox.Conf.Static.Listeners {
 		var tlsConfig *tls.Config
 		if listener.TLS != nil {
@@ -311,44 +311,57 @@ func ListenAndServe() {
 		if listener.IMAP.Enabled {
 			port := config.Port(listener.IMAP.Port, 143)
 			for _, ip := range listener.IPs {
-				go listenServe("imap", name, ip, port, tlsConfig, false, listener.IMAP.NoRequireSTARTTLS)
+				listen1("imap", name, ip, port, tlsConfig, false, listener.IMAP.NoRequireSTARTTLS)
 			}
 		}
 
 		if listener.IMAPS.Enabled {
 			port := config.Port(listener.IMAPS.Port, 993)
 			for _, ip := range listener.IPs {
-				go listenServe("imaps", name, ip, port, tlsConfig, true, false)
+				listen1("imaps", name, ip, port, tlsConfig, true, false)
 			}
 		}
 	}
 }
 
-func listenServe(protocol, listenerName, ip string, port int, tlsConfig *tls.Config, xtls, noRequireSTARTTLS bool) {
+var servers []func()
+
+func listen1(protocol, listenerName, ip string, port int, tlsConfig *tls.Config, xtls, noRequireSTARTTLS bool) {
 	addr := net.JoinHostPort(ip, fmt.Sprintf("%d", port))
-	xlog.Print("listening for imap", mlog.Field("listener", listenerName), mlog.Field("addr", addr), mlog.Field("protocol", protocol))
+	if os.Getuid() == 0 {
+		xlog.Print("listening for imap", mlog.Field("listener", listenerName), mlog.Field("addr", addr), mlog.Field("protocol", protocol))
+	}
 	network := mox.Network(ip)
-	var ln net.Listener
-	var err error
-	if xtls {
-		ln, err = tls.Listen(network, addr, tlsConfig)
-	} else {
-		ln, err = net.Listen(network, addr)
-	}
+	ln, err := mox.Listen(network, addr)
 	if err != nil {
-		xlog.Fatalx("imap: listen for imap"+mox.LinuxSetcapHint(err), err, mlog.Field("protocol", protocol), mlog.Field("listener", listenerName))
+		xlog.Fatalx("imap: listen for imap", err, mlog.Field("protocol", protocol), mlog.Field("listener", listenerName))
+	}
+	if xtls {
+		ln = tls.NewListener(ln, tlsConfig)
 	}
 
-	for {
-		conn, err := ln.Accept()
-		if err != nil {
-			xlog.Infox("imap: accept", err, mlog.Field("protocol", protocol), mlog.Field("listener", listenerName))
-			continue
+	serve := func() {
+		for {
+			conn, err := ln.Accept()
+			if err != nil {
+				xlog.Infox("imap: accept", err, mlog.Field("protocol", protocol), mlog.Field("listener", listenerName))
+				continue
+			}
+
+			metricIMAPConnection.WithLabelValues(protocol).Inc()
+			go serve(listenerName, mox.Cid(), tlsConfig, conn, xtls, noRequireSTARTTLS)
 		}
-
-		metricIMAPConnection.WithLabelValues(protocol).Inc()
-		go serve(listenerName, mox.Cid(), tlsConfig, conn, xtls, noRequireSTARTTLS)
 	}
+
+	servers = append(servers, serve)
+}
+
+// Serve starts serving on all listeners, launching a goroutine per listener.
+func Serve() {
+	for _, serve := range servers {
+		go serve()
+	}
+	servers = nil
 }
 
 // returns whether this connection accepts utf-8 in strings.
@@ -605,6 +618,7 @@ func serve(listenerName string, cid int64, tlsConfig *tls.Config, nc net.Conn, x
 	})
 	c.tr = moxio.NewTraceReader(c.log, "C: ", c.conn)
 	c.tw = moxio.NewTraceWriter(c.log, "S: ", c)
+	// todo: tracing should be done on whatever comes out of c.br. the remote connection write a command plus data, and bufio can read it in one read, causing a command parser that sets the tracing level to data to have no effect. we are now typically logging sent messages, when mail clients append to the Sent mailbox.
 	c.br = bufio.NewReader(c.tr)
 	c.bw = bufio.NewWriter(c.tw)
 
@@ -1277,7 +1291,8 @@ func (c *conn) cmdCapability(tag, cmd string, p *parser) {
 func (c *conn) capabilities() string {
 	caps := serverCapabilities
 	// ../rfc/9051:1238
-	if !c.tls {
+	// We only allow starting without TLS when explicitly configured, in violation of RFC.
+	if !c.tls && c.tlsConfig != nil {
 		caps += " STARTTLS"
 	}
 	if c.tls || c.noRequireSTARTTLS {
@@ -2952,7 +2967,11 @@ func (c *conn) cmdxCopy(isUID bool, tag, cmd string, p *parser) {
 				m.ID = 0
 				m.UID = uidFirst + store.UID(i)
 				m.MailboxID = mbDst.ID
-				m.MailboxOrigID = mbSrc.ID
+				if mbSrc.Name == conf.RejectsMailbox && m.MailboxDestinedID != 0 {
+					// Incorrectly delivered to Rejects mailbox. Adjust MailboxOrigID so this message
+					// is used for reputation calculation during future deliveries.
+					m.MailboxOrigID = m.MailboxDestinedID
+				}
 				m.TrainedJunk = nil
 				m.JunkFlagsForMailbox(mbDst.Name, conf)
 				err := tx.Insert(&m)
@@ -3077,7 +3096,7 @@ func (c *conn) cmdxMove(isUID bool, tag, cmd string, p *parser) {
 
 	c.account.WithWLock(func() {
 		c.xdbwrite(func(tx *bstore.Tx) {
-			c.xmailboxID(tx, c.mailboxID) // Validate.
+			mbSrc := c.xmailboxID(tx, c.mailboxID) // Validate.
 			mbDst = c.xmailbox(tx, name, "TRYCREATE")
 			if mbDst.ID == c.mailboxID {
 				xuserErrorf("cannot move to currently selected mailbox")
@@ -3113,6 +3132,11 @@ func (c *conn) cmdxMove(isUID bool, tag, cmd string, p *parser) {
 					xserverErrorf("internal error: got uid %d, expected %d, for index %d", m.UID, uids[i], i)
 				}
 				m.MailboxID = mbDst.ID
+				if mbSrc.Name == conf.RejectsMailbox && m.MailboxDestinedID != 0 {
+					// Incorrectly delivered to Rejects mailbox. Adjust MailboxOrigID so this message
+					// is used for reputation calculation during future deliveries.
+					m.MailboxOrigID = m.MailboxDestinedID
+				}
 				m.UID = uidnext
 				m.JunkFlagsForMailbox(mbDst.Name, conf)
 				uidnext++
