@@ -718,14 +718,14 @@ func (f *Filter) TrainDirs(hamDir, sentDir, spamDir string, hamFiles, sentFiles,
 
 	fields := []mlog.Pair{
 		mlog.Field("hams", hams),
-		mlog.Field("hamTime", tham),
-		mlog.Field("hamMalformed", hamMalformed),
+		mlog.Field("hamtime", tham),
+		mlog.Field("hammalformed", hamMalformed),
 		mlog.Field("sent", sent),
-		mlog.Field("sentTime", tsent),
-		mlog.Field("sentMalformed", sentMalformed),
+		mlog.Field("senttime", tsent),
+		mlog.Field("sentmalformed", sentMalformed),
 		mlog.Field("spams", f.spams),
-		mlog.Field("spamTime", tspam),
-		mlog.Field("spamMalformed", spamMalformed),
+		mlog.Field("spamtime", tspam),
+		mlog.Field("spammalformed", spamMalformed),
 		mlog.Field("dbsize", fmt.Sprintf("%.1fmb", float64(dbSize)/(1024*1024))),
 		mlog.Field("bloomsize", fmt.Sprintf("%.1fmb", float64(bloomSize)/(1024*1024))),
 		mlog.Field("bloom1ratio", fmt.Sprintf("%.4f", float64(f.bloom.Ones())/float64(len(f.bloom.Bytes())*8))),
