@@ -78,7 +78,6 @@ var commands = []struct {
 }{
 	{"serve", cmdServe},
 	{"quickstart", cmdQuickstart},
-	{"restart", cmdRestart},
 	{"stop", cmdStop},
 	{"setaccountpassword", cmdSetaccountpassword},
 	{"setadminpassword", cmdSetadminpassword},
@@ -105,6 +104,8 @@ var commands = []struct {
 	{"config domain add", cmdConfigDomainAdd},
 	{"config domain rm", cmdConfigDomainRemove},
 	{"config describe-sendmail", cmdConfigDescribeSendmail},
+	{"config printservice", cmdConfigPrintservice},
+	{"example", cmdExample},
 
 	{"checkupdate", cmdCheckupdate},
 	{"cid", cmdCid},
@@ -144,6 +145,7 @@ var commands = []struct {
 	{"updates addsigned", cmdUpdatesAddSigned},
 	{"updates genkey", cmdUpdatesGenkey},
 	{"updates pubkey", cmdUpdatesPubkey},
+	{"updates serve", cmdUpdatesServe},
 	{"updates verify", cmdUpdatesVerify},
 }
 
@@ -353,7 +355,7 @@ var loglevel string
 // restores any loglevel specified on the command-line, instead of using the
 // loglevels from the config file.
 func mustLoadConfig() {
-	mox.MustLoadConfig()
+	mox.MustLoadConfig(false)
 	if level, ok := mlog.Levels[loglevel]; ok && loglevel != "" {
 		mox.Conf.Log[""] = level
 		mlog.SetConfig(mox.Conf.Log)
@@ -476,7 +478,7 @@ are printed.
 		c.Usage()
 	}
 
-	_, errs := mox.ParseConfig(context.Background(), mox.ConfigStaticPath, true)
+	_, errs := mox.ParseConfig(context.Background(), mox.ConfigStaticPath, true, false, false)
 	if len(errs) > 1 {
 		log.Printf("multiple errors:")
 		for _, err := range errs {
@@ -530,6 +532,27 @@ needs modifications to make it valid.
 	xcheckf(err, "describing config")
 }
 
+func cmdConfigPrintservice(c *cmd) {
+	c.params = ">mox.service"
+	c.help = `Prints a systemd unit service file for mox.
+
+This is the same file as generated using quickstart. If the systemd service file
+has changed with a newer version of mox, use this command to generate an up to
+date version.
+`
+	if len(c.Parse()) != 0 {
+		c.Usage()
+	}
+
+	pwd, err := os.Getwd()
+	if err != nil {
+		log.Printf("current working directory: %v", err)
+		pwd = "/home/mox"
+	}
+	service := strings.ReplaceAll(moxService, "/home/mox", pwd)
+	fmt.Print(service)
+}
+
 func cmdConfigDomainAdd(c *cmd) {
 	c.params = "domain account [localpart]"
 	c.help = `Adds a new domain to the configuration and reloads the configuration.
@@ -737,6 +760,115 @@ func cmdConfigDNSCheck(c *cmd) {
 	printResult("Autodiscover", result.Autodiscover.Result)
 }
 
+var examples = []struct {
+	Name string
+	Get  func() string
+}{
+	{
+		"webhandlers",
+		func() string {
+			const webhandlers = `# Snippet of domains.conf to configure WebDomainRedirects and WebHandlers.
+
+# Redirect all requests for mox.example to https://www.mox.example.
+WebDomainRedirects:
+	mox.example: www.mox.example
+
+# Each request is matched against these handlers until one matches and serves it.
+WebHandlers:
+	-
+		# The name of the handler, used in logging and metrics.
+		LogName: staticmjl
+		# With ACME configured, each configured domain will automatically get a TLS
+		# certificate on first request.
+		Domain: www.mox.example
+		PathRegexp: ^/who/mjl/
+		WebStatic:
+			StripPrefix: /who/mjl
+			# Requested path /who/mjl/inferno/ resolves to local web/mjl/inferno.
+			# If a directory contains an index.html, it is served when a directory is requested.
+			Root: web/mjl
+			# With ListFiles true, if a directory does not contain an index.html, the contents are listed.
+			ListFiles: true
+			ResponseHeaders:
+				X-Mox: hi
+	-
+		LogName: redir
+		Domain: www.mox.example
+		PathRegexp: ^/redir/a/b/c
+		# Don't redirect from plain HTTP to HTTPS.
+		DontRedirectPlainHTTP: true
+		WebRedirect:
+			# Just change the domain and add query string set fragment. No change to scheme.
+			# Path will start with /redir/a/b/c (and whathever came after) because no
+			# OrigPathRegexp+ReplacePath is set.
+			BaseURL: //moxest.example?q=1#frag
+			# Default redirection is 308 - Permanent Redirect.
+			StatusCode: 307
+	-
+		LogName: oldnew
+		Domain: www.mox.example
+		PathRegexp: ^/old/
+		WebRedirect:
+			# Replace path, leaving rest of URL intact.
+			OrigPathRegexp: ^/old/(.*)
+			ReplacePath: /new/$1
+	-
+		LogName: app
+		Domain: www.mox.example
+		PathRegexp: ^/app/
+		WebForward:
+			# Strip the path matched by PathRegexp before forwarding the request. So original
+			# request /app/api become just /api.
+			StripPath: true
+			# URL of backend, where requests are forwarded to. The path in the URL is kept,
+			# so for incoming request URL /app/api, the outgoing request URL has path /app-v2/api.
+			# Requests are made with Go's net/http DefaultTransporter, including using
+			# HTTP_PROXY and HTTPS_PROXY environment variables.
+			URL: http://127.0.0.1:8900/app-v2/
+			# Add headers to response.
+			ResponseHeaders:
+				X-Frame-Options: deny
+				X-Content-Type-Options: nosniff
+`
+			// Parse just so we know we have the syntax right.
+			// todo: ideally we would have a complete config file and parse it fully.
+			var conf struct {
+				WebDomainRedirects map[string]string
+				WebHandlers        []config.WebHandler
+			}
+			err := sconf.Parse(strings.NewReader(webhandlers), &conf)
+			xcheckf(err, "parsing webhandlers example")
+			return webhandlers
+		},
+	},
+}
+
+func cmdExample(c *cmd) {
+	c.params = "[name]"
+	c.help = `List available examples, or print a specific example.`
+
+	args := c.Parse()
+	if len(args) > 1 {
+		c.Usage()
+	}
+
+	var match func() string
+	for _, ex := range examples {
+		if len(args) == 0 {
+			fmt.Println(ex.Name)
+		} else if args[0] == ex.Name {
+			match = ex.Get
+		}
+	}
+	if len(args) == 0 {
+		return
+	}
+	if match == nil {
+		log.Fatalln("not found")
+	}
+	fmt.Print(match())
+}
+
 func cmdLoglevels(c *cmd) {
 	c.params = "[level [pkg]]"
 	c.help = `Print the log levels, or set a new default log level, or a level for the given package.
@@ -802,42 +934,6 @@ new mail deliveries.
 	fmt.Println("mox stopped")
 }
 
-func cmdRestart(c *cmd) {
-	c.help = `Restart mox after validating the configuration file.
-
-Restart execs the mox binary, which have been updated. Restart returns after
-the restart has finished. If you update the mox binary, keep in mind that the
-validation of the configuration file is done by the old process with the old
-binary. The new binary may report a syntax error. If you update the binary, you
-should use the "config test" command with the new binary to validate the
-configuration file.
-
-Like stop, existing connections get a 3 second period for graceful shutdown.
-`
-	if len(c.Parse()) != 0 {
-		c.Usage()
-	}
-	mustLoadConfig()
-
-	ctl := xctl()
-	ctl.xwrite("restart")
-	line := ctl.xread()
-	if line != "ok" {
-		log.Fatalf("restart failed: %s", line)
-	}
-	// Server is now restarting. It will write ok when it is back online again. If it fails, our connection will be closed.
-	buf := make([]byte, 128)
-	n, err := ctl.conn.Read(buf)
-	if err != nil {
-		log.Fatalf("restart failed: %s", err)
-	}
-	s := strings.TrimSuffix(string(buf[:n]), "\n")
-	if s != "ok" {
-		log.Fatalf("restart failed: %s", s)
-	}
-	fmt.Println("mox restarted")
-}
-
 func cmdSetadminpassword(c *cmd) {
 	c.help = `Set a new admin password, for the web interface.
 
@@ -862,7 +958,16 @@ The password is read from stdin. Its bcrypt hash is stored in a file named
 }
 
 func xreadpassword() string {
-	fmt.Println("Type new password. Password WILL echo.")
+	fmt.Printf(`
+Type new password. Password WILL echo.
+
+WARNING: Bots will try to bruteforce your password. Connections with failed
+authentication attempts will be rate limited but attackers WILL find weak
+passwords. If your account is compromised, spammers are likely to abuse your
+system, spamming your address and the wider internet in your name. So please
+pick a random, unguessable password, preferrably at least 12 characters.
+
+`)
 	fmt.Printf("password: ")
 	buf := make([]byte, 64)
 	n, err := os.Stdin.Read(buf)
