@@ -11,10 +11,13 @@ import (
 	"errors"
 	"fmt"
 	"net"
+	"net/url"
 	"os"
+	"os/user"
 	"path/filepath"
 	"regexp"
 	"sort"
+	"strconv"
 	"strings"
 	"sync"
 	"time"
@@ -136,6 +139,7 @@ func (c *Config) loadDynamic() []error {
 	c.Dynamic = d
 	c.dynamicMtime = mtime
 	c.accountDestinations = accDests
+	c.allowACMEHosts(true)
 	return nil
 }
 
@@ -194,38 +198,57 @@ func (c *Config) AccountDestination(addr string) (accDests AccountDestination, o
 	return
 }
 
-func (c *Config) allowACMEHosts() {
-	// todo future: reset the allowed hosts for autoconfig & mtasts when loading new list.
+func (c *Config) WebServer() (r map[dns.Domain]dns.Domain, l []config.WebHandler) {
+	c.withDynamicLock(func() {
+		r = c.Dynamic.WebDNSDomainRedirects
+		l = c.Dynamic.WebHandlers
+	})
+	return r, l
+}
+
+func (c *Config) allowACMEHosts(checkACMEHosts bool) {
 	for _, l := range c.Static.Listeners {
 		if l.TLS == nil || l.TLS.ACME == "" {
 			continue
 		}
-		m := c.Static.ACME[l.TLS.ACME].Manager
-		for _, dom := range c.Dynamic.Domains {
 
-			if l.AutoconfigHTTPS.Enabled {
+		m := c.Static.ACME[l.TLS.ACME].Manager
+		hostnames := map[dns.Domain]struct{}{}
+
+		hostnames[c.Static.HostnameDomain] = struct{}{}
+		if l.HostnameDomain.ASCII != "" {
+			hostnames[l.HostnameDomain] = struct{}{}
+		}
+
+		for _, dom := range c.Dynamic.Domains {
+			if l.AutoconfigHTTPS.Enabled && !l.AutoconfigHTTPS.NonTLS {
 				if d, err := dns.ParseDomain("autoconfig." + dom.Domain.ASCII); err != nil {
 					xlog.Errorx("parsing autoconfig domain", err, mlog.Field("domain", dom.Domain))
 				} else {
-					m.AllowHostname(d)
-				}
-
-				if d, err := dns.ParseDomain("autodiscover." + dom.Domain.ASCII); err != nil {
-					xlog.Errorx("parsing autodiscover domain", err, mlog.Field("domain", dom.Domain))
-				} else {
-					m.AllowHostname(d)
+					hostnames[d] = struct{}{}
 				}
 			}
 
-			if l.MTASTSHTTPS.Enabled && dom.MTASTS != nil {
+			if l.MTASTSHTTPS.Enabled && dom.MTASTS != nil && !l.MTASTSHTTPS.NonTLS {
 				d, err := dns.ParseDomain("mta-sts." + dom.Domain.ASCII)
 				if err != nil {
 					xlog.Errorx("parsing mta-sts domain", err, mlog.Field("domain", dom.Domain))
-					continue
+				} else {
+					hostnames[d] = struct{}{}
 				}
-				m.AllowHostname(d)
 			}
 		}
+
+		if l.WebserverHTTPS.Enabled {
+			for from := range c.Dynamic.WebDNSDomainRedirects {
+				hostnames[from] = struct{}{}
+			}
+			for _, wh := range c.Dynamic.WebHandlers {
+				hostnames[wh.DNSDomain] = struct{}{}
+			}
+		}
+
+		m.SetAllowedHostnames(dns.StrictResolver{Pkg: "autotls"}, hostnames, c.Static.Listeners["public"].IPs, checkACMEHosts)
 	}
 }
 
@@ -282,17 +305,17 @@ func writeDynamic(ctx context.Context, log *mlog.Log, c config.Dynamic) error {
 	Conf.Dynamic = c
 	Conf.accountDestinations = accDests
 
-	Conf.allowACMEHosts()
+	Conf.allowACMEHosts(true)
 
 	return nil
 }
 
 // MustLoadConfig loads the config, quitting on errors.
-func MustLoadConfig() {
+func MustLoadConfig(checkACMEHosts bool) {
 	Shutdown, ShutdownCancel = context.WithCancel(context.Background())
 	Context, ContextCancel = context.WithCancel(context.Background())
 
-	errs := LoadConfig(context.Background())
+	errs := LoadConfig(context.Background(), checkACMEHosts)
 	if len(errs) > 1 {
 		xlog.Error("loading config file: multiple errors")
 		for _, err := range errs {
@@ -306,8 +329,8 @@ func MustLoadConfig() {
 
 // LoadConfig attempts to parse and load a config, returning any errors
 // encountered.
-func LoadConfig(ctx context.Context) []error {
-	c, errs := ParseConfig(ctx, ConfigStaticPath, false)
+func LoadConfig(ctx context.Context, checkACMEHosts bool) []error {
+	c, errs := ParseConfig(ctx, ConfigStaticPath, false, false, checkACMEHosts)
 	if len(errs) > 0 {
 		return errs
 	}
@@ -323,9 +346,13 @@ func SetConfig(c *Config) {
 	Conf = Config{c.Static, sync.Mutex{}, c.Log, sync.Mutex{}, c.Dynamic, c.dynamicMtime, c.DynamicLastCheck, c.accountDestinations}
 }
 
-// ParseConfig parses the static config at path p. If checkOnly is true, no
-// changes are made, such as registering ACME identities.
-func ParseConfig(ctx context.Context, p string, checkOnly bool) (c *Config, errs []error) {
+// ParseConfig parses the static config at path p. If checkOnly is true, no changes
+// are made, such as registering ACME identities. If skipCheckTLSKeyCerts is true,
+// the TLS KeyCerts configuration is not checked. This is used during the
+// quickstart in the case the user is going to provide their own certificates.
+// If checkACMEHosts is true, the hosts allowed for acme are compared with the
+// explicitly configured ips we are listening on.
+func ParseConfig(ctx context.Context, p string, checkOnly, skipCheckTLSKeyCerts, checkACMEHosts bool) (c *Config, errs []error) {
 	c = &Config{
 		Static: config.Static{
 			DataDir: ".",
@@ -344,7 +371,7 @@ func ParseConfig(ctx context.Context, p string, checkOnly bool) (c *Config, errs
 		return nil, []error{fmt.Errorf("parsing %s: %v", p, err)}
 	}
 
-	if xerrs := PrepareStaticConfig(ctx, p, c, checkOnly); len(xerrs) > 0 {
+	if xerrs := PrepareStaticConfig(ctx, p, c, checkOnly, skipCheckTLSKeyCerts); len(xerrs) > 0 {
 		return nil, xerrs
 	}
 
@@ -352,7 +379,7 @@ func ParseConfig(ctx context.Context, p string, checkOnly bool) (c *Config, errs
 	c.Dynamic, c.dynamicMtime, c.accountDestinations, errs = ParseDynamicConfig(ctx, pp, c.Static)
 
 	if !checkOnly {
-		c.allowACMEHosts()
+		c.allowACMEHosts(checkACMEHosts)
 	}
 
 	return c, errs
@@ -361,7 +388,7 @@ func ParseConfig(ctx context.Context, p string, checkOnly bool) (c *Config, errs
 // PrepareStaticConfig parses the static config file and prepares data structures
 // for starting mox. If checkOnly is set no substantial changes are made, like
 // creating an ACME registration.
-func PrepareStaticConfig(ctx context.Context, configFile string, config *Config, checkOnly bool) (errs []error) {
+func PrepareStaticConfig(ctx context.Context, configFile string, config *Config, checkOnly, skipCheckTLSKeyCerts bool) (errs []error) {
 	addErrorf := func(format string, args ...any) {
 		errs = append(errs, fmt.Errorf(format, args...))
 	}
@@ -391,6 +418,35 @@ func PrepareStaticConfig(ctx context.Context, configFile string, config *Config,
 		}
 	}
 
+	if c.User == "" {
+		c.User = "mox"
+	}
+	u, err := user.Lookup(c.User)
+	var userErr user.UnknownUserError
+	if err != nil && errors.As(err, &userErr) {
+		uid, err := strconv.ParseUint(c.User, 10, 32)
+		if err != nil {
+			addErrorf("parsing unknown user %s as uid: %v (hint: add user mox with \"useradd -d $PWD mox\" or specify a different username on the quickstart command-line)", c.User, err)
+		} else {
+			// We assume the same gid as uid.
+			c.UID = uint32(uid)
+			c.GID = uint32(uid)
+		}
+	} else if err != nil {
+		addErrorf("looking up user: %v", err)
+	} else {
+		if uid, err := strconv.ParseUint(u.Uid, 10, 32); err != nil {
+			addErrorf("parsing uid %s: %v", u.Uid, err)
+		} else {
+			c.UID = uint32(uid)
+		}
+		if gid, err := strconv.ParseUint(u.Gid, 10, 32); err != nil {
+			addErrorf("parsing gid %s: %v", u.Gid, err)
+		} else {
+			c.GID = uint32(gid)
+		}
+	}
+
 	hostname, err := dns.ParseDomain(c.Hostname)
 	if err != nil {
 		addErrorf("parsing hostname: %s", err)
@@ -431,10 +487,10 @@ func PrepareStaticConfig(ctx context.Context, configFile string, config *Config,
 					addErrorf("listener %q: unknown ACME provider %q", name, l.TLS.ACME)
 				}
 
-				// If only checking, we don't have an acme manager, so set an empty tls config to
-				// continue without errors.
+				// If only checking or with missing ACME definition, we don't have an acme manager,
+				// so set an empty tls config to continue.
 				var tlsconfig *tls.Config
-				if checkOnly {
+				if checkOnly || acme.Manager == nil {
 					tlsconfig = &tls.Config{}
 				} else {
 					tlsconfig = acme.Manager.TLSConfig.Clone()
@@ -456,8 +512,10 @@ func PrepareStaticConfig(ctx context.Context, configFile string, config *Config,
 				}
 				l.TLS.Config = tlsconfig
 			} else if len(l.TLS.KeyCerts) != 0 {
-				if err := loadTLSKeyCerts(configFile, "listener "+name, l.TLS); err != nil {
-					addErrorf("%w", err)
+				if !skipCheckTLSKeyCerts {
+					if err := loadTLSKeyCerts(configFile, "listener "+name, l.TLS); err != nil {
+						addErrorf("%w", err)
+					}
 				}
 			} else {
 				addErrorf("listener %q: cannot have TLS config without ACME and without static keys/certificates", name)
@@ -484,11 +542,28 @@ func PrepareStaticConfig(ctx context.Context, configFile string, config *Config,
 			if l.TLS.ACMEConfig != nil {
 				l.TLS.ACMEConfig.MinVersion = minVersion
 			}
-		} else if l.IMAPS.Enabled || l.SMTP.Enabled && !l.SMTP.NoSTARTTLS || l.Submissions.Enabled || l.Submission.Enabled && !l.Submission.NoRequireSTARTTLS || l.AccountHTTPS.Enabled || l.AdminHTTPS.Enabled || l.AutoconfigHTTPS.Enabled || l.MTASTSHTTPS.Enabled {
-			addErrorf("listener %q requires TLS, but does not specify tls config", name)
+		} else {
+			var needsTLS []string
+			needtls := func(s string, v bool) {
+				if v {
+					needsTLS = append(needsTLS, s)
+				}
+			}
+			needtls("IMAPS", l.IMAPS.Enabled)
+			needtls("SMTP", l.SMTP.Enabled && !l.SMTP.NoSTARTTLS)
+			needtls("Submissions", l.Submissions.Enabled)
+			needtls("Submission", l.Submission.Enabled && !l.Submission.NoRequireSTARTTLS)
+			needtls("AccountHTTPS", l.AccountHTTPS.Enabled)
+			needtls("AdminHTTPS", l.AdminHTTPS.Enabled)
+			needtls("AutoconfigHTTPS", l.AutoconfigHTTPS.Enabled && !l.AutoconfigHTTPS.NonTLS)
+			needtls("MTASTSHTTPS", l.MTASTSHTTPS.Enabled && !l.MTASTSHTTPS.NonTLS)
+			needtls("WebserverHTTPS", l.WebserverHTTPS.Enabled)
+			if len(needsTLS) > 0 {
+				addErrorf("listener %q does not specify tls config, but requires tls for %s", name, strings.Join(needsTLS, ", "))
+			}
 		}
-		if l.AutoconfigHTTPS.Enabled && (!l.IMAP.Enabled && !l.IMAPS.Enabled || !l.Submission.Enabled && !l.Submissions.Enabled) {
-			addErrorf("listener %q with autoconfig enabled must have SMTP submission or submissions and IMAP or IMAPS enabled", name)
+		if l.AutoconfigHTTPS.Enabled && l.MTASTSHTTPS.Enabled && l.AutoconfigHTTPS.Port == l.MTASTSHTTPS.Port && l.AutoconfigHTTPS.NonTLS != l.MTASTSHTTPS.NonTLS {
+			addErrorf("listener %q tries to enable autoconfig and mta-sts enabled on same port but with both http and https", name)
 		}
 		if l.SMTP.Enabled {
 			if len(l.IPs) == 0 {
@@ -604,11 +679,13 @@ func prepareDynamicConfig(ctx context.Context, dynamicPath string, static config
 	}
 	checkMailboxNormf(static.Postmaster.Mailbox, "postmaster mailbox")
 
-	var haveSTSListener bool
+	var haveSTSListener, haveWebserverListener bool
 	for _, l := range static.Listeners {
 		if l.MTASTSHTTPS.Enabled {
 			haveSTSListener = true
-			break
+		}
+		if l.WebserverHTTP.Enabled || l.WebserverHTTPS.Enabled {
+			haveWebserverListener = true
 		}
 	}
 
@@ -915,6 +992,123 @@ func prepareDynamicConfig(ctx context.Context, dynamicPath string, static config
 		checkMailboxNormf(tlsrpt.Mailbox, "TLSRPT mailbox for account %q", tlsrpt.Account)
 		accDests[addrFull] = AccountDestination{lp, tlsrpt.Account, dest}
 	}
+
+	// Check webserver configs.
+	if (len(c.WebDomainRedirects) > 0 || len(c.WebHandlers) > 0) && !haveWebserverListener {
+		addErrorf("WebDomainRedirects or WebHandlers configured but no listener with WebserverHTTP or WebserverHTTPS enabled")
+	}
+
+	c.WebDNSDomainRedirects = map[dns.Domain]dns.Domain{}
+	for from, to := range c.WebDomainRedirects {
+		fromdom, err := dns.ParseDomain(from)
+		if err != nil {
+			addErrorf("parsing domain for redirect %s: %v", from, err)
+		}
+		todom, err := dns.ParseDomain(to)
+		if err != nil {
+			addErrorf("parsing domain for redirect %s: %v", to, err)
+		} else if fromdom == todom {
+			addErrorf("will not redirect domain %s to itself", todom)
+		}
+		var zerodom dns.Domain
+		if _, ok := c.WebDNSDomainRedirects[fromdom]; ok && fromdom != zerodom {
+			addErrorf("duplicate redirect domain %s", from)
+		}
+		c.WebDNSDomainRedirects[fromdom] = todom
+	}
+
+	for i := range c.WebHandlers {
+		wh := &c.WebHandlers[i]
+
+		if wh.LogName == "" {
+			wh.Name = fmt.Sprintf("%d", i)
+		} else {
+			wh.Name = wh.LogName
+		}
+
+		dom, err := dns.ParseDomain(wh.Domain)
+		if err != nil {
+			addErrorf("webhandler %s %s: parsing domain: %v", wh.Domain, wh.PathRegexp, err)
+		}
+		wh.DNSDomain = dom
+
+		if !strings.HasPrefix(wh.PathRegexp, "^") {
+			addErrorf("webhandler %s %s: path regexp must start with a ^", wh.Domain, wh.PathRegexp)
+		}
+		re, err := regexp.Compile(wh.PathRegexp)
+		if err != nil {
+			addErrorf("webhandler %s %s: compiling regexp: %v", wh.Domain, wh.PathRegexp, err)
+		}
+		wh.Path = re
+
+		var n int
+		if wh.WebStatic != nil {
+			n++
+			ws := wh.WebStatic
+			if ws.StripPrefix != "" && !strings.HasPrefix(ws.StripPrefix, "/") {
+				addErrorf("webstatic %s %s: prefix to strip %s must start with a slash", wh.Domain, wh.PathRegexp, ws.StripPrefix)
+			}
+			for k := range ws.ResponseHeaders {
+				xk := k
+				k := strings.TrimSpace(xk)
+				if k != xk || k == "" {
+					addErrorf("webstatic %s %s: bad header %q", wh.Domain, wh.PathRegexp, xk)
+				}
+			}
+		}
+		if wh.WebRedirect != nil {
+			n++
+			wr := wh.WebRedirect
+			if wr.BaseURL != "" {
+				u, err := url.Parse(wr.BaseURL)
+				if err != nil {
+					addErrorf("webredirect %s %s: parsing redirect url %s: %v", wh.Domain, wh.PathRegexp, wr.BaseURL, err)
+				}
+				switch u.Path {
+				case "", "/":
+					u.Path = "/"
+				default:
+					addErrorf("webredirect %s %s: BaseURL must have empty path", wh.Domain, wh.PathRegexp, wr.BaseURL)
+				}
+				wr.URL = u
+			}
+			if wr.OrigPathRegexp != "" && wr.ReplacePath != "" {
+				re, err := regexp.Compile(wr.OrigPathRegexp)
+				if err != nil {
+					addErrorf("webredirect %s %s: compiling regexp %s: %v", wh.Domain, wh.PathRegexp, wr.OrigPathRegexp, err)
+				}
+				wr.OrigPath = re
+			} else if wr.OrigPathRegexp != "" || wr.ReplacePath != "" {
+				addErrorf("webredirect %s %s: must have either both OrigPathRegexp and ReplacePath, or neither", wh.Domain, wh.PathRegexp)
+			} else if wr.BaseURL == "" {
+				addErrorf("webredirect %s %s: must at least one of BaseURL and OrigPathRegexp+ReplacePath", wh.Domain, wh.PathRegexp)
+			}
+			if wr.StatusCode != 0 && (wr.StatusCode < 300 || wr.StatusCode >= 400) {
+				addErrorf("webredirect %s %s: invalid redirect status code %d", wh.Domain, wh.PathRegexp, wr.StatusCode)
+			}
+		}
+		if wh.WebForward != nil {
+			n++
+			wf := wh.WebForward
+			u, err := url.Parse(wf.URL)
+			if err != nil {
+				addErrorf("webforward %s %s: parsing url %s: %v", wh.Domain, wh.PathRegexp, wf.URL, err)
+			}
+			wf.TargetURL = u
+
+			for k := range wf.ResponseHeaders {
+				xk := k
+				k := strings.TrimSpace(xk)
+				if k != xk || k == "" {
+					addErrorf("webforward %s %s: bad header %q", wh.Domain, wh.PathRegexp, xk)
+				}
+			}
+		}
+		if n != 1 {
+			addErrorf("webhandler %s %s: must have exactly one handler, not %d", wh.Domain, wh.PathRegexp, n)
+		}
+	}
+
 	return
 }
 
