@@ -161,8 +161,9 @@ var (
 
 var jitterRand = mox.NewRand()
 
-// ListenAndServe starts network listeners that serve incoming SMTP connection.
-func ListenAndServe() {
+// Listen initializes network listeners for incoming SMTP connection.
+// The listeners are stored for a later call to Serve.
+func Listen() {
 	for name, listener := range mox.Conf.Static.Listeners {
 		var tlsConfig *tls.Config
 		if listener.TLS != nil {
@@ -181,7 +182,7 @@ func ListenAndServe() {
 			}
 			port := config.Port(listener.SMTP.Port, 25)
 			for _, ip := range listener.IPs {
-				go listenServe("smtp", name, ip, port, hostname, tlsConfig, false, false, maxMsgSize, false, listener.SMTP.RequireSTARTTLS, listener.SMTP.DNSBLZones)
+				listen1("smtp", name, ip, port, hostname, tlsConfig, false, false, maxMsgSize, false, listener.SMTP.RequireSTARTTLS, listener.SMTP.DNSBLZones)
 			}
 		}
 		if listener.Submission.Enabled {
@@ -191,7 +192,7 @@ func ListenAndServe() {
 			}
 			port := config.Port(listener.Submission.Port, 587)
 			for _, ip := range listener.IPs {
-				go listenServe("submission", name, ip, port, hostname, tlsConfig, true, false, maxMsgSize, !listener.Submission.NoRequireSTARTTLS, !listener.Submission.NoRequireSTARTTLS, nil)
+				listen1("submission", name, ip, port, hostname, tlsConfig, true, false, maxMsgSize, !listener.Submission.NoRequireSTARTTLS, !listener.Submission.NoRequireSTARTTLS, nil)
 			}
 		}
 
@@ -202,34 +203,47 @@ func ListenAndServe() {
 			}
 			port := config.Port(listener.Submissions.Port, 465)
 			for _, ip := range listener.IPs {
-				go listenServe("submissions", name, ip, port, hostname, tlsConfig, true, true, maxMsgSize, true, true, nil)
+				listen1("submissions", name, ip, port, hostname, tlsConfig, true, true, maxMsgSize, true, true, nil)
 			}
 		}
 	}
 }
 
-func listenServe(protocol, name, ip string, port int, hostname dns.Domain, tlsConfig *tls.Config, submission, xtls bool, maxMessageSize int64, requireTLSForAuth, requireTLSForDelivery bool, dnsBLs []dns.Domain) {
+var servers []func()
+
+func listen1(protocol, name, ip string, port int, hostname dns.Domain, tlsConfig *tls.Config, submission, xtls bool, maxMessageSize int64, requireTLSForAuth, requireTLSForDelivery bool, dnsBLs []dns.Domain) {
 	addr := net.JoinHostPort(ip, fmt.Sprintf("%d", port))
-	xlog.Print("listening for smtp", mlog.Field("listener", name), mlog.Field("address", addr), mlog.Field("protocol", protocol))
+	if os.Getuid() == 0 {
+		xlog.Print("listening for smtp", mlog.Field("listener", name), mlog.Field("address", addr), mlog.Field("protocol", protocol))
+	}
 	network := mox.Network(ip)
-	var ln net.Listener
-	var err error
-	if xtls {
-		ln, err = tls.Listen(network, addr, tlsConfig)
-	} else {
-		ln, err = net.Listen(network, addr)
-	}
+	ln, err := mox.Listen(network, addr)
 	if err != nil {
-		xlog.Fatalx("smtp: listen for smtp"+mox.LinuxSetcapHint(err), err, mlog.Field("protocol", protocol), mlog.Field("listener", name))
+		xlog.Fatalx("smtp: listen for smtp", err, mlog.Field("protocol", protocol), mlog.Field("listener", name))
 	}
-	for {
-		conn, err := ln.Accept()
-		if err != nil {
-			xlog.Infox("smtp: accept", err, mlog.Field("protocol", protocol), mlog.Field("listener", name))
-			continue
+	if xtls {
+		ln = tls.NewListener(ln, tlsConfig)
+	}
+
+	serve := func() {
+		for {
+			conn, err := ln.Accept()
+			if err != nil {
+				xlog.Infox("smtp: accept", err, mlog.Field("protocol", protocol), mlog.Field("listener", name))
+				continue
+			}
+			resolver := dns.StrictResolver{} // By leaving Pkg empty, it'll be set by each package that uses the resolver, e.g. spf/dkim/dmarc.
+			go serve(name, mox.Cid(), hostname, tlsConfig, conn, resolver, submission, xtls, maxMessageSize, requireTLSForAuth, requireTLSForDelivery, dnsBLs)
 		}
-		resolver := dns.StrictResolver{} // By leaving Pkg empty, it'll be set by each package that uses the resolver, e.g. spf/dkim/dmarc.
-		go serve(name, mox.Cid(), hostname, tlsConfig, conn, resolver, submission, xtls, maxMessageSize, requireTLSForAuth, requireTLSForDelivery, dnsBLs)
+	}
+
+	servers = append(servers, serve)
+}
+
+// Serve starts serving on all listeners, launching a goroutine per listener.
+func Serve() {
+	for _, serve := range servers {
+		go serve()
 	}
 }
 
@@ -422,7 +436,7 @@ var bufpool = moxio.NewBufpool(8, 2*1024)
 func (c *conn) readline() string {
 	line, err := bufpool.Readline(c.r)
 	if err != nil && errors.Is(err, moxio.ErrLineTooLong) {
-		c.writecodeline(smtp.C500BadSyntax, smtp.SeProto5Other0, "line too line, smtp max is 512, we reached 2048", nil)
+		c.writecodeline(smtp.C500BadSyntax, smtp.SeProto5Other0, "line too long, smtp max is 512, we reached 2048", nil)
 		panic(fmt.Errorf("%s (%w)", err, errIO))
 	} else if err != nil {
 		panic(fmt.Errorf("%s (%w)", err, errIO))
@@ -654,7 +668,7 @@ func command(c *conn) {
 
 		var serr smtpError
 		if errors.As(err, &serr) {
-			c.writecodeline(serr.code, serr.secode, serr.err.Error(), serr.err)
+			c.writecodeline(serr.code, serr.secode, fmt.Sprintf("%s (%s)", serr.err, mox.ReceivedID(c.cid)), serr.err)
 			if serr.printStack {
 				debug.PrintStack()
 			}
@@ -1443,7 +1457,7 @@ func (c *conn) cmdData(p *parser) {
 			if n < defaultMaxMsgSize {
 				ecode = smtp.SeMailbox2MsgLimitExceeded3
 			}
-			c.writecodeline(smtp.C451LocalErr, ecode, "error copying data to file", err)
+			c.writecodeline(smtp.C451LocalErr, ecode, fmt.Sprintf("error copying data to file (%s)", mox.ReceivedID(c.cid)), err)
 			panic(fmt.Errorf("remote sent too much DATA: %w", errIO))
 		}
 
@@ -1453,7 +1467,7 @@ func (c *conn) cmdData(p *parser) {
 		// available and our write blocks us from reading remaining data, leading to
 		// deadlock. We have a timeout on our connection writes though, so worst case we'll
 		// abort the connection due to expiration.
-		c.writecodeline(smtp.C451LocalErr, smtp.SeSys3Other0, "error copying data to file", err)
+		c.writecodeline(smtp.C451LocalErr, smtp.SeSys3Other0, fmt.Sprintf("error copying data to file (%s)", mox.ReceivedID(c.cid)), err)
 		io.Copy(io.Discard, dr)
 		return
 	}
@@ -1874,7 +1888,7 @@ func (c *conn) deliver(ctx context.Context, recvHdrFor func(string) string, msgW
 	})
 	switch receivedSPF.Result {
 	case spf.StatusPass:
-		c.log.Debug("spf pass", mlog.Field("ip", spfArgs.RemoteIP), mlog.Field("mailfromDomain", spfArgs.MailFromDomain.ASCII)) // todo: log the domain that was actually verified.
+		c.log.Debug("spf pass", mlog.Field("ip", spfArgs.RemoteIP), mlog.Field("mailfromdomain", spfArgs.MailFromDomain.ASCII)) // todo: log the domain that was actually verified.
 	case spf.StatusFail:
 		if spfExpl != "" {
 			// Filter out potentially hostile text. ../rfc/7208:2529
@@ -2237,7 +2251,7 @@ func (c *conn) deliver(ctx context.Context, recvHdrFor func(string) string, msgW
 			conf, _ := acc.Conf()
 			if conf.RejectsMailbox != "" && messageID != "" {
 				if err := acc.RejectsRemove(log, conf.RejectsMailbox, messageID); err != nil {
-					log.Errorx("removing message from rejects mailbox", err, mlog.Field("messageID", messageID))
+					log.Errorx("removing message from rejects mailbox", err, mlog.Field("messageid", messageID))
 				}
 			}
 		})
