@@ -229,11 +229,23 @@ type Message struct {
 	UID       UID   `bstore:"nonzero"` // UID, for IMAP. Set during deliver.
 	MailboxID int64 `bstore:"nonzero,unique MailboxID+UID,index MailboxID+Received,ref Mailbox"`
 
-	// Mailbox message originally delivered to. I.e. not changed when moved to Trash or
-	// Junk. Useful for per-mailbox reputation. Not a bstore reference to prevent
-	// having to update all messages in a mailbox when the original mailbox is removed.
-	// Use of this field requires checking if the mailbox still exists.
-	MailboxOrigID int64
+	// MailboxOrigID is the mailbox the message was originally delivered to. Typically
+	// Inbox or Rejects, but can also be Postmaster and TLS/DMARC reporting addresses.
+	// MailboxOrigID is not changed when the message is moved to another mailbox, e.g.
+	// Archive/Trash/Junk. Used for per-mailbox reputation.
+	//
+	// MailboxDestinedID is normally 0, but when a message is delivered to the Rejects
+	// mailbox, it is set to the intended mailbox according to delivery rules,
+	// typically that of Inbox. When such a message is moved out of Rejects, the
+	// MailboxOrigID is corrected by setting it to MailboxDestinedID. This ensures the
+	// message is used for reputation calculation for future deliveries to that
+	// mailbox.
+	//
+	// These are not bstore references to prevent having to update all messages in a
+	// mailbox when the original mailbox is removed. Use of these fields requires
+	// checking if the mailbox still exists.
+	MailboxOrigID     int64
+	MailboxDestinedID int64
 
 	Received time.Time `bstore:"default now,index"`
 
@@ -600,6 +612,11 @@ func (a *Account) DeliverX(log *mlog.Log, tx *bstore.Tx, m *Message, msgFile *os
 		m.ParsedBuf = buf
 	}
 
+	// If we are delivering to the originally intended mailbox, no need to store the mailbox ID again.
+	if m.MailboxDestinedID != 0 && m.MailboxDestinedID == m.MailboxOrigID {
+		m.MailboxDestinedID = 0
+	}
+
 	err = tx.Insert(m)
 	xcheckf(err, "inserting message")
 
@@ -671,14 +688,12 @@ func (a *Account) DeliverX(log *mlog.Log, tx *bstore.Tx, m *Message, msgFile *os
 		xcheckf(err, "sync directory")
 	}
 
-	if notrain && m.NeedsTraining() {
-		// If this ever happens, hopefully we'll get bug reports about it.
-		log.Error("deliver of message that unexpectedly needs training", mlog.Field("messageid", m.ID), mlog.Field("trainedjunk", m.TrainedJunk), mlog.Field("flags", m.Flags))
+	if !notrain && m.NeedsTraining() {
+		l := []Message{*m}
+		err = a.RetrainMessages(log, tx, l, false)
+		xcheckf(err, "training junkfilter")
+		*m = l[0]
 	}
-	l := []Message{*m}
-	err = a.RetrainMessages(log, tx, l, false)
-	xcheckf(err, "training junkfilter")
-	*m = l[0]
 }
 
 // write contents of r to new file dst, for delivering a message.
@@ -1186,25 +1201,29 @@ func (a *Account) RejectsRemove(log *mlog.Log, rejectsMailbox, messageID string)
 }
 
 // We keep a cache of recent successful authentications, so we don't have to bcrypt successful calls each time.
-var authCache struct {
+var authCache = struct {
 	sync.Mutex
 	success map[authKey]string
+}{
+	success: map[authKey]string{},
 }
 
 type authKey struct {
 	email, hash string
 }
 
-func init() {
-	authCache.success = map[authKey]string{}
-	go func() {
-		for {
-			authCache.Lock()
-			authCache.success = map[authKey]string{}
-			authCache.Unlock()
-			time.Sleep(15 * time.Minute)
-		}
-	}()
+// StartAuthCache starts a goroutine that regularly clears the auth cache.
+func StartAuthCache() {
+	go manageAuthCache()
+}
+
+func manageAuthCache() {
+	for {
+		authCache.Lock()
+		authCache.success = map[authKey]string{}
+		authCache.Unlock()
+		time.Sleep(15 * time.Minute)
+	}
 }
 
 // OpenEmailAuth opens an account given an email address and password.
